@@ -4,6 +4,24 @@
  * Class for localization update hooks and static methods.
  */
 class LocalisationUpdate {
+	/**
+	 * Hook: LocalisationCacheRecacheFallback
+	 */
+	public static function onRecacheFallback( LocalisationCache $lc, $code, array &$cache ) {
+		$dir = LocalisationUpdate::getDirectory();
+		if ( !$dir ) {
+			return true;
+		}
+
+		$fileName = "$dir/" . self::getFilename( $code );
+		if ( is_readable( $fileName ) ) {
+			$data = FormatJson::decode( file_get_contents( $fileName ), true );
+			$cache['messages'] = array_merge( $cache['messages'], $data );
+		}
+
+		return true;
+	}
+
 	/**
 	 * Hook: LocalisationCacheRecache
 	 */
@@ -13,14 +31,9 @@ class LocalisationUpdate {
 			return true;
 		}
 
-		$codeSequence = array_merge( array( $code ), $cache['fallbackSequence'] );
+		$codeSequence = array_merge( [ $code ], $cache['fallbackSequence'] );
 		foreach ( $codeSequence as $csCode ) {
 			$fileName = "$dir/" . self::getFilename( $csCode );
-			if ( is_readable( $fileName ) ) {
-				$data = FormatJson::decode( file_get_contents( $fileName ), true );
-				$cache['messages'] = array_merge( $cache['messages'], $data );
-			}
-
 			$cache['deps'][] = new FileDependency( $fileName );
 		}
 
@@ -36,10 +49,7 @@ class LocalisationUpdate {
 	public static function getDirectory() {
 		global $wgLocalisationUpdateDirectory, $wgCacheDirectory;
 
-		// ?: can be used once we drop support for MW 1.19
-		return $wgLocalisationUpdateDirectory ?
-			$wgLocalisationUpdateDirectory :
-			$wgCacheDirectory;
+		return $wgLocalisationUpdateDirectory ?: $wgCacheDirectory;
 	}
 
 	/**
