@@ -498,6 +498,50 @@ class SyntaxHighlight_GeSHi {
 		return true;
 	}
 
+	/**
+	 * Conditionally register resource loader modules that depends on the
+	 * VisualEditor MediaWiki extension.
+	 *
+	 * @param $resourceLoader
+	 * @return true
+	 */
+	public static function onResourceLoaderRegisterModules( &$resourceLoader ) {
+		if ( ! ExtensionRegistry::getInstance()->isLoaded( 'VisualEditor' ) ) {
+			return;
+		}
+
+		$resourceLoader->register( 'ext.geshi.visualEditor', [
+			'class' => 'ResourceLoaderGeSHiVisualEditorModule',
+			'localBasePath' => __DIR__ . DIRECTORY_SEPARATOR . 'modules',
+			'remoteExtPath' => 'SyntaxHighlight_GeSHi/modules',
+			'scripts' => [
+				've-syntaxhighlight/ve.dm.MWSyntaxHighlightNode.js',
+				've-syntaxhighlight/ve.ce.MWSyntaxHighlightNode.js',
+				've-syntaxhighlight/ve.ui.MWSyntaxHighlightWindow.js',
+				've-syntaxhighlight/ve.ui.MWSyntaxHighlightDialog.js',
+				've-syntaxhighlight/ve.ui.MWSyntaxHighlightDialogTool.js',
+				've-syntaxhighlight/ve.ui.MWSyntaxHighlightInspector.js',
+				've-syntaxhighlight/ve.ui.MWSyntaxHighlightInspectorTool.js',
+			],
+			'styles' => [
+				've-syntaxhighlight/ve.ce.MWSyntaxHighlightNode.css',
+				've-syntaxhighlight/ve.ui.MWSyntaxHighlightDialog.css',
+				've-syntaxhighlight/ve.ui.MWSyntaxHighlightInspector.css',
+			],
+			'dependencies' => [
+				'ext.visualEditor.mwcore',
+			],
+			'messages' => [
+				'syntaxhighlight-visualeditor-mwsyntaxhighlightinspector-code',
+				'syntaxhighlight-visualeditor-mwsyntaxhighlightinspector-language',
+				'syntaxhighlight-visualeditor-mwsyntaxhighlightinspector-none',
+				'syntaxhighlight-visualeditor-mwsyntaxhighlightinspector-showlines',
+				'syntaxhighlight-visualeditor-mwsyntaxhighlightinspector-title',
+			],
+			'targets' => [ 'desktop', 'mobile' ],
+		] );
+	}
+
 	/** Backward-compatibility shim for extensions.  */
 	public static function prepare( $text, $lang ) {
 		wfDeprecated( __METHOD__ );
