@@ -20,6 +20,7 @@
  * @file
  * @author Aaron Schulz
  */
+use Psr\Log\LoggerInterface;
 
 /**
  * Class to handle job queues stored in Redis
@@ -66,6 +67,8 @@
 class JobQueueRedis extends JobQueue {
 	/** @var RedisConnectionPool */
 	protected $redisPool;
+	/** @var LoggerInterface */
+	protected $logger;
 
 	/** @var string Server address */
 	protected $server;
@@ -96,6 +99,7 @@ class JobQueueRedis extends JobQueue {
 				"Non-daemonized mode is no longer supported. Please install the " .
 				"mediawiki/services/jobrunner service and update \$wgJobTypeConf as needed." );
 		}
+		$this->logger = \MediaWiki\Logger\LoggerFactory::getInstance( 'redis' );
 	}
 
 	protected function supportedOrders() {
@@ -250,6 +254,7 @@ class JobQueueRedis extends JobQueue {
 			$args[] = (string)$this->serialize( $item );
 		}
 		static $script =
+		/** @lang Lua */
 <<<LUA
 		local kUnclaimed, kSha1ById, kIdBySha1, kDelayed, kData, kQwJobs = unpack(KEYS)
 		-- First argument is the queue ID
@@ -339,6 +344,7 @@ LUA;
 	 */
 	protected function popAndAcquireBlob( RedisConnRef $conn ) {
 		static $script =
+		/** @lang Lua */
 <<<LUA
 		local kUnclaimed, kSha1ById, kIdBySha1, kClaimed, kAttempts, kData = unpack(KEYS)
 		local rTime = unpack(ARGV)
@@ -386,6 +392,7 @@ LUA;
 		$conn = $this->getConnection();
 		try {
 			static $script =
+			/** @lang Lua */
 <<<LUA
 			local kClaimed, kAttempts, kData = unpack(KEYS)
 			local id = unpack(ARGV)
@@ -745,7 +752,7 @@ LUA;
 	 * @throws JobQueueConnectionError
 	 */
 	protected function getConnection() {
-		$conn = $this->redisPool->getConnection( $this->server );
+		$conn = $this->redisPool->getConnection( $this->server, $this->logger );
 		if ( !$conn ) {
 			throw new JobQueueConnectionError(
 				"Unable to connect to redis server {$this->server}." );
