@@ -51,10 +51,6 @@ class SpecialBotPasswords extends FormSpecialPage {
 		return $this->getConfig()->get( 'EnableBotPasswords' );
 	}
 
-	protected function getLoginSecurityLevel() {
-		return $this->getName();
-	}
-
 	/**
 	 * Main execution point
 	 * @param string|null $par
@@ -111,9 +107,6 @@ class SpecialBotPasswords extends FormSpecialPage {
 					'type' => 'check',
 					'label-message' => 'botpasswords-label-resetpassword',
 				];
-				if ( $this->botPassword->isInvalid() ) {
-					$fields['resetPassword']['default'] = true;
-				}
 			}
 
 			$lang = $this->getLanguage();
@@ -153,58 +146,29 @@ class SpecialBotPasswords extends FormSpecialPage {
 			];
 
 			$fields['restrictions'] = [
-				'type' => 'textarea',
-				'label-message' => 'botpasswords-label-restrictions',
+				'class' => 'HTMLRestrictionsField',
 				'required' => true,
-				'default' => $this->botPassword->getRestrictions()->toJson( true ),
-				'rows' => 5,
-				'validation-callback' => function ( $v ) {
-					try {
-						MWRestrictions::newFromJson( $v );
-						return true;
-					} catch ( InvalidArgumentException $ex ) {
-						return $ex->getMessage();
-					}
-				},
+				'default' => $this->botPassword->getRestrictions(),
 			];
 
 		} else {
-			$passwordFactory = new PasswordFactory();
-			$passwordFactory->init( $this->getConfig() );
-
+			$linkRenderer = $this->getLinkRenderer();
 			$dbr = BotPassword::getDB( DB_REPLICA );
 			$res = $dbr->select(
 				'bot_passwords',
-				[ 'bp_app_id', 'bp_password' ],
+				[ 'bp_app_id' ],
 				[ 'bp_user' => $this->userId ],
 				__METHOD__
 			);
 			foreach ( $res as $row ) {
-				try {
-					$password = $passwordFactory->newFromCiphertext( $row->bp_password );
-					$passwordInvalid = $password instanceof InvalidPassword;
-					unset( $password );
-				} catch ( PasswordError $ex ) {
-					$passwordInvalid = true;
-				}
-
-				$text = Linker::link(
-					$this->getPageTitle( $row->bp_app_id ),
-					htmlspecialchars( $row->bp_app_id ),
-					[],
-					[],
-					[ 'known' ]
-				);
-				if ( $passwordInvalid ) {
-					$text .= $this->msg( 'word-separator' )->escaped()
-						. $this->msg( 'botpasswords-label-needsreset' )->parse();
-				}
-
 				$fields[] = [
 					'section' => 'existing',
 					'type' => 'info',
 					'raw' => true,
-					'default' => $text,
+					'default' => $linkRenderer->makeKnownLink(
+						$this->getPageTitle( $row->bp_app_id ),
+						$row->bp_app_id
+					),
 				];
 			}
 
@@ -260,7 +224,7 @@ class SpecialBotPasswords extends FormSpecialPage {
 					'name' => 'op',
 					'value' => 'create',
 					'label-message' => 'botpasswords-label-create',
-					'flags' => [ 'primary', 'constructive' ],
+					'flags' => [ 'primary', 'progressive' ],
 				] );
 			}
 
@@ -308,7 +272,7 @@ class SpecialBotPasswords extends FormSpecialPage {
 		$bp = BotPassword::newUnsaved( [
 			'centralId' => $this->userId,
 			'appId' => $this->par,
-			'restrictions' => MWRestrictions::newFromJson( $data['restrictions'] ),
+			'restrictions' => $data['restrictions'],
 			'grants' => array_merge(
 				MWGrants::getHiddenGrants(),
 				preg_replace( '/^grant-/', '', $data['grants'] )
