@@ -67,8 +67,8 @@ class BotPassword implements IDBAccessObject {
 
 	/**
 	 * Get a database connection for the bot passwords database
-	 * @param int $db Index of the connection to get, e.g. DB_MASTER or DB_SLAVE.
-	 * @return DatabaseBase
+	 * @param int $db Index of the connection to get, e.g. DB_MASTER or DB_REPLICA.
+	 * @return Database
 	 */
 	public static function getDB( $db ) {
 		global $wgBotPasswordsCluster, $wgBotPasswordsDatabase;
@@ -258,15 +258,6 @@ class BotPassword implements IDBAccessObject {
 		}
 	}
 
-	/**
-	 * Whether the password is currently invalid
-	 * @since 1.32
-	 * @return bool
-	 */
-	public function isInvalid() {
-		return $this->getPassword() instanceof InvalidPassword;
-	}
-
 	/**
 	 * Save the BotPassword to the database
 	 * @param string $operation 'update' or 'insert'
@@ -470,10 +461,6 @@ class BotPassword implements IDBAccessObject {
 			return Status::newFatal( 'nosuchuser', $name );
 		}
 
-		if ( $user->isLocked() ) {
-			return Status::newFatal( 'botpasswords-locked' );
-		}
-
 		// Throttle
 		$throttle = null;
 		if ( !empty( $wgPasswordAttemptThrottle ) ) {
@@ -501,11 +488,7 @@ class BotPassword implements IDBAccessObject {
 		}
 
 		// Check the password
-		$passwordObj = $bp->getPassword();
-		if ( $passwordObj instanceof InvalidPassword ) {
-			return Status::newFatal( 'botpasswords-needs-reset', $name, $appId );
-		}
-		if ( !$passwordObj->equals( $password ) ) {
+		if ( !$bp->getPassword()->equals( $password ) ) {
 			return Status::newFatal( 'wrongpassword' );
 		}
 
