@@ -1,8 +1,7 @@
 <?php
 /** Lingala (lingála)
  *
- * See MessagesQqq.php for message documentation incl. usage of parameters
- * To improve a translation please visit http://translatewiki.net
+ * To improve a translation please visit https://translatewiki.net
  *
  * @ingroup Language
  * @file
@@ -15,10 +14,10 @@
 $fallback = 'fr';
 
 // Remove French aliases
-$namespaceGenderAliases = array();
+$namespaceGenderAliases = [];
 
 $linkPrefixExtension = true;
 
 # Same as the French (bug 8485)
-$separatorTransformTable = array( ',' => "\xc2\xa0", '.' => ',' );
+$separatorTransformTable = [ ',' => "\xc2\xa0", '.' => ',' ];
 
