@@ -63,7 +63,8 @@ if ( $ext == 'php' || $ext == 'php5' ) {
 	return true;
 }
 $mime = false;
-$lines = explode( "\n", file_get_contents( "includes/mime.types" ) );
+// Borrow mime type file from MimeAnalyzer
+$lines = explode( "\n", file_get_contents( "includes/libs/mime/mime.types" ) );
 foreach ( $lines as $line ) {
 	$exts = explode( " ", $line );
 	$mime = array_shift( $exts );
