@@ -30,7 +30,7 @@ class SearchEngineTest extends MediaWikiLangTestCase {
 			$this->markTestSkipped( "MySQL or SQLite with FTS3 only" );
 		}
 
-		$searchType = $this->db->getSearchEngine();
+		$searchType = SearchEngineFactory::getSearchEngineClass( $this->db );
 		$this->setMwGlobals( [
 			'wgSearchType' => $searchType
 		] );
@@ -50,6 +50,10 @@ class SearchEngineTest extends MediaWikiLangTestCase {
 			return;
 		}
 
+		// Reset the search type back to default - some extensions may have
+		// overridden it.
+		$this->setMwGlobals( [ 'wgSearchType' => null ] );
+
 		$this->insertPage( 'Not_Main_Page', 'This is not a main page' );
 		$this->insertPage(
 			'Talk:Not_Main_Page',
@@ -209,4 +213,100 @@ class SearchEngineTest extends MediaWikiLangTestCase {
 			"Title power search failed" );
 	}
 
+	/**
+	 * @covers SearchEngine::getSearchIndexFields
+	 */
+	public function testSearchIndexFields() {
+		/**
+		 * @var $mockEngine SearchEngine
+		 */
+		$mockEngine = $this->getMock( 'SearchEngine', [ 'makeSearchFieldMapping' ] );
+
+		$mockFieldBuilder = function ( $name, $type ) {
+			$mockField =
+				$this->getMockBuilder( 'SearchIndexFieldDefinition' )->setConstructorArgs( [
+					$name,
+					$type
+				] )->getMock();
+
+			$mockField->expects( $this->any() )->method( 'getMapping' )->willReturn( [
+				'testData' => 'test',
+				'name' => $name,
+				'type' => $type,
+			] );
+
+			$mockField->expects( $this->any() )
+				->method( 'merge' )
+				->willReturn( $mockField );
+
+			return $mockField;
+		};
+
+		$mockEngine->expects( $this->atLeastOnce() )
+			->method( 'makeSearchFieldMapping' )
+			->willReturnCallback( $mockFieldBuilder );
+
+		// Not using mock since PHPUnit mocks do not work properly with references in params
+		$this->setTemporaryHook( 'SearchIndexFields',
+			function ( &$fields, SearchEngine $engine ) use ( $mockFieldBuilder ) {
+				$fields['testField'] =
+					$mockFieldBuilder( "testField", SearchIndexField::INDEX_TYPE_TEXT );
+				return true;
+			} );
+
+		$fields = $mockEngine->getSearchIndexFields();
+		$this->assertArrayHasKey( 'language', $fields );
+		$this->assertArrayHasKey( 'category', $fields );
+		$this->assertInstanceOf( 'SearchIndexField', $fields['testField'] );
+
+		$mapping = $fields['testField']->getMapping( $mockEngine );
+		$this->assertArrayHasKey( 'testData', $mapping );
+		$this->assertEquals( 'test', $mapping['testData'] );
+	}
+
+	public function hookSearchIndexFields( $mockFieldBuilder, &$fields, SearchEngine $engine ) {
+		$fields['testField'] = $mockFieldBuilder( "testField", SearchIndexField::INDEX_TYPE_TEXT );
+		return true;
+	}
+
+	public function testAugmentorSearch() {
+		$this->search->setNamespaces( [ 0, 1, 4 ] );
+		$resultSet = $this->search->searchText( 'smithee' );
+		// Not using mock since PHPUnit mocks do not work properly with references in params
+		$this->mergeMwGlobalArrayValue( 'wgHooks',
+			[ 'SearchResultsAugment' => [ [ $this, 'addAugmentors' ] ] ] );
+		$this->search->augmentSearchResults( $resultSet );
+		for ( $result = $resultSet->next(); $result; $result = $resultSet->next() ) {
+			$id = $result->getTitle()->getArticleID();
+			$augmentData = "Result:$id:" . $result->getTitle()->getText();
+			$augmentData2 = "Result2:$id:" . $result->getTitle()->getText();
+			$this->assertEquals( [ 'testSet' => $augmentData, 'testRow' => $augmentData2 ],
+				$result->getExtensionData() );
+		}
+	}
+
+	public function addAugmentors( &$setAugmentors, &$rowAugmentors ) {
+		$setAugmentor = $this->getMock( 'ResultSetAugmentor' );
+		$setAugmentor->expects( $this->once() )
+			->method( 'augmentAll' )
+			->willReturnCallback( function ( SearchResultSet $resultSet ) {
+				$data = [];
+				for ( $result = $resultSet->next(); $result; $result = $resultSet->next() ) {
+					$id = $result->getTitle()->getArticleID();
+					$data[$id] = "Result:$id:" . $result->getTitle()->getText();
+				}
+				$resultSet->rewind();
+				return $data;
+			} );
+		$setAugmentors['testSet'] = $setAugmentor;
+
+		$rowAugmentor = $this->getMock( 'ResultAugmentor' );
+		$rowAugmentor->expects( $this->exactly( 2 ) )
+			->method( 'augment' )
+			->willReturnCallback( function ( SearchResult $result ) {
+				$id = $result->getTitle()->getArticleID();
+				return "Result2:$id:" . $result->getTitle()->getText();
+			} );
+		$rowAugmentors['testRow'] = $rowAugmentor;
+	}
 }
